import * as vscode from 'vscode';
import axios from 'axios';
import { exec } from 'child_process';

// Funktion zur Authentifizierung bei Gitea mit dem Personal Access Token (PAT)
async function authenticateGitea() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return;
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            vscode.window.showInformationMessage(`Authentifizierung erfolgreich: ${response.data.username}`);
        }
    } catch (error) {
        vscode.window.showErrorMessage('Authentifizierung fehlgeschlagen.');
        console.error(error);
    }
}

// Funktion zur Konfiguration der Gitea-Instanz und des Tokens über die Command Palette
async function configureGitea() {
    const instanceUrl = await vscode.window.showInputBox({
        prompt: "Gitea Instanz-URL eingeben",
        placeHolder: "https://your-gitea-instance.com"
    });

    const token = await vscode.window.showInputBox({
        prompt: "Gitea Personal Access Token eingeben",
        placeHolder: "token",
        password: true
    });

    if (instanceUrl && token) {
        await vscode.workspace.getConfiguration().update('gitea.instanceUrl', instanceUrl, vscode.ConfigurationTarget.Global);
        await vscode.workspace.getConfiguration().update('gitea.personalAccessToken', token, vscode.ConfigurationTarget.Global);
        vscode.window.showInformationMessage('Gitea Konfiguration aktualisiert.');
    }
}

// Funktion zum Abrufen der Repositories des Benutzers von Gitea
async function getGiteaRepositories(): Promise<any[]> {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage('Gitea URL oder Token ist nicht konfiguriert.');
        return [];
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user/repos`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            return response.data;
        } else {
            vscode.window.showErrorMessage('Fehler beim Abrufen der Repositories.');
            return [];
        }
    } catch (error) {
        vscode.window.showErrorMessage('Fehler bei der Verbindung mit der Gitea API.');
        console.error(error);
        return [];
    }
}

// Funktion zum Klonen eines Repositories via SSH
async function cloneGiteaRepository() {
    const repos = await getGiteaRepositories();

    if (repos.length === 0) {
        vscode.window.showInformationMessage('Keine Repositories gefunden.');
        return;
    }

    const repoNames = repos.map(repo => repo.full_name);
    const selectedRepo = await vscode.window.showQuickPick(repoNames, {
        placeHolder: 'Wähle ein Repository zum Klonen aus'
    });

    if (!selectedRepo) {
        vscode.window.showInformationMessage('Kein Repository ausgewählt.');
        return;
    }

    const repo = repos.find(r => r.full_name === selectedRepo);

    // Verwende die SSH-URL anstelle von HTTPS
    if (repo && repo.ssh_url) {
        const folderUri = await vscode.window.showOpenDialog({
            canSelectFiles: false,
            canSelectFolders: true,
            canSelectMany: false,
            openLabel: 'Ordner zum Klonen auswählen'
        });

        if (folderUri && folderUri[0]) {
            const folderPath = folderUri[0].fsPath;

            vscode.window.withProgress({
                location: vscode.ProgressLocation.Window,
                title: `Cloning ${selectedRepo}`,
                cancellable: false
            }, async (progress) => {
                progress.report({ message: 'Klonvorgang läuft...' });

                exec(`git clone ${repo.ssh_url} ${folderPath}`, (error, stdout, stderr) => {
                    if (error) {
                        vscode.window.showErrorMessage('Fehler beim Klonen des Repositories.');
                        console.error(stderr);
                    } else {
                        vscode.window.showInformationMessage(`Repository ${selectedRepo} erfolgreich geklont.`);
                        console.log(stdout);
                    }
                });
            });
        } else {
            vscode.window.showInformationMessage('Kein Zielordner ausgewählt.');
        }
    } else {
        vscode.window.showErrorMessage('Konnte die SSH-Klon-URL nicht finden.');
    }
}

// Aktivierungsfunktion des Plugins
export function activate(context: vscode.ExtensionContext) {
    // Registriert den Befehl zur Authentifizierung
    let authCommand = vscode.commands.registerCommand('gitea.authenticate', authenticateGitea);
    context.subscriptions.push(authCommand);

    // Registriert den Befehl zur Konfiguration
    let configCommand = vscode.commands.registerCommand('gitea.configure', configureGitea);
    context.subscriptions.push(configCommand);

	// Befehl zum Klonen eines Repositories
    let cloneCommand = vscode.commands.registerCommand('gitea.cloneRepository', cloneGiteaRepository);
    context.subscriptions.push(cloneCommand);
}

// Deaktivierungsfunktion des Plugins
export function deactivate() {}
